﻿// Copyright (c) 2020 Tudor Stanciu

using MediatR;
using Microsoft.Extensions.Logging;
using System.Threading;
using System.Threading.Tasks;
using Tuitio.Application.Services.Abstractions;
using Tuitio.PublishedLanguage.Constants;
using Tuitio.PublishedLanguage.Dto;

namespace Tuitio.Application.CommandHandlers
{
    public class AccountLoginHandler
    {
        public record Command(string UserName, string Password) : IRequest<Envelope<AccountLoginResult>>;

        public class CommandHandler : IRequestHandler<Command, Envelope<AccountLoginResult>>
        {
            private readonly IUserService _userService;
            private readonly ILogger<AccountLoginHandler> _logger;

            public CommandHandler(IUserService userService, ILogger<AccountLoginHandler> logger)
            {
                _userService = userService;
                _logger = logger;
            }

            public async Task<Envelope<AccountLoginResult>> Handle(Command command, CancellationToken cancellationToken)
            {
                var loginResult = await _userService.Login(command.UserName, command.Password);
                if (loginResult == null)
                {
                    _logger.LogDebug($"Login failed for user '{command.UserName}'.");
                    return Envelope<AccountLoginResult>.Fail(EnvelopeError.BAD_CREDENTIALS);
                }

                _logger.LogDebug($"Login succeeded for user '{command.UserName}'.");

                var result = new AccountLoginResult(loginResult.Raw, loginResult.Token.ExpiresIn);
                return Envelope<AccountLoginResult>.Success(result);
            }
        }
    }
}
