﻿// Copyright (c) 2020 Tudor Stanciu

using AutoMapper;
using MediatR;
using Microsoft.Extensions.Logging;
using System.Threading;
using System.Threading.Tasks;
using Tuitio.Application.Services.Abstractions;
using Tuitio.PublishedLanguage.Constants;
using Tuitio.PublishedLanguage.Dto;

namespace Tuitio.Application.CommandHandlers
{
    public class AuthorizationHandler
    {
        public record Command(string Token) : IRequest<Envelope<AuthorizationResult>>;

        public class CommandHandler : IRequestHandler<Command, Envelope<AuthorizationResult>>
        {
            private readonly IUserService _userService;
            private readonly IMapper _mapper;
            private readonly ILogger<AuthorizationHandler> _logger;

            public CommandHandler(IUserService userService, IMapper mapper, ILogger<AuthorizationHandler> logger)
            {
                _userService = userService;
                _mapper = mapper;
                _logger = logger;
            }

            public Task<Envelope<AuthorizationResult>> Handle(Command command, CancellationToken cancellationToken)
            {
                var token = _userService.Authorize(command.Token);
                if (token == null)
                {
                    _logger.LogDebug($"Authorization failed for token '{command.Token}'.");
                    var result = Envelope<AuthorizationResult>.Fail(EnvelopeError.UNAUTHORIZED);
                    return Task.FromResult(result);
                }

                _logger.LogDebug($"Authorization succeeded for token '{command.Token}'.");
                var authorizationResult = _mapper.Map<AuthorizationResult>(token);
                var envelope = Envelope<AuthorizationResult>.Success(authorizationResult);
                return Task.FromResult(envelope);
            }
        }
    }
}
